package samples;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static java.lang.System.out;

/**
 * Some more advanced techniques for List
 */
public class AdvancedUseOFList {


    void program() {

        // Simplest (!!) way to initialize a List
        // Arrays.asList() will create an unmodifiable list
        // To create a modifiable list we must create another ArrayList taking the values
        // from the immutable list
        List<Integer> list = new ArrayList<>(Arrays.asList(1, 2, 3, 4, 5, 6));


        // Safe way to remove many using short for loop
        List<Integer> tmp = new ArrayList<>();
        for (Integer i : list) {
            if (i < 4) {
                tmp.add(i);
            }
        }
        list.removeAll(tmp);


        List<Point> points = new ArrayList<>(Arrays.asList(new Point(2, 3), new Point(1, 5)
                , new Point(-1, 3), new Point(4, -6)));

        // Quick way to alter list (we don't assign the reference, would be useless)
        for (Point p : points) {
            p.x++;
        }

        // Or even quicker, use a method reference
        points.forEach(this::incX);
        out.println(points);

    }

    class Point {
        int x;
        int y;

        public Point(int x, int y) {
            this.x = x;
            this.y = y;
        }
    }

    void incX(Point point) {
        point.x++;
    }
}
