package samples.oo.collections;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static java.lang.System.out;

/*
 *  Collections and inheritance
 *
 *  If S is sub type of T then List<S> is NOT subtype of List<T>
 *
 *  Cat > Animal         --->   Animal a =  new Cat();     OK!
 *
 *                       --->   List<Cat> cats = new ArrayList...
 *                              List<Animal> as = cats;    NO!
 *
 *  but S[] is subtype of T[]
 *  Cat > Animal         --->   Cat[] cats = new ...
 *                              Animal[] as = cats;     // Ok!
 *
 */
public class CollectionAndInherit {

    public static void main(String[] args) {
        new CollectionAndInherit().program();
    }

    List<Animal> animals = new ArrayList<>(Arrays.asList(new Cat("Missan", 3, false), new Dog("Fido", 4)));

    void program() {
        //List<ISayable> sayables = animals;   // No! ...

        /*
        List<Dog> dogs = new ArrayList<>();     // The reason
        List<Animal> animals = dogs;             // Animal :> Dog , but NOT List<Animal :> List<Dog>
        animals.add(new Cat("Grållan",2, false));
        Dog dog = dogs.get(0);
        */

        out.println(getSaybles());

        // The array loop hole (hole in type system)
        /*
        String[] strings = new String[2];
        Object[] objects = strings;  //  Object :> String and Object[] :> String[]
        objects[0] = 12; // Runtime exception
        */

    }

    // Convert list of Animals to list of ISayables (reducing information)
    public List<ISayable> getSaybles() {
        List<ISayable> tmp = new ArrayList<>();
        tmp.addAll(animals);
        return tmp;
    }

      /*
    public List<ISayable> getSaybles() {
        //return animals;
        //return (ISayable>) animals;
        return Collections.unmodifiableList(animals);  // Not allowed to add anything, then ok!
    }*/


}
